/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "StrategyFactory.hh"
#include "IStrategy.hh"
#include "AcceptStrategy.hh"
#include "DeclineStrategy.hh"
#include "VariableStrategy.hh"

// Konstruktor.
StrategyFactory::StrategyFactory()
{
    // Array mit allen moeglichen Strategien.
    mStrategies.insert( std::pair<std::string,IStrategy*>("accept",   new AcceptStrategy()) );
    mStrategies.insert( std::pair<std::string,IStrategy*>("decline",  new DeclineStrategy()) );
    mStrategies.insert( std::pair<std::string,IStrategy*>("variable", new VariableStrategy()) );
}

// Gibt Instanz der Klasse zurueck.
StrategyFactory& StrategyFactory::getInstance()
{
    // erstelle einmalig die Klasse
    static StrategyFactory factory;
    return factory;
}

// Erstellt eine bestimmte Strategie.
IStrategy* StrategyFactory::create( const std::string& id )
{
    IStrategy* strategyP = 0;

    // Suche Strategie in Array.
    if ( mStrategies.count( id ) != 0 )
    {
        // Achtung: Wir duerfen nicht einfach den Pointer
        // kopieren, da wir sonst immer das gleiche Objekt
        // nach aussen geben wuerden. Daher muessen wir ein
        // komplett neues Objekt erstellen.
        if ( mStrategies[id] )
        {
            strategyP = mStrategies[id]->copy();
        }
    }

    return strategyP;
}
